unit J2HUtils;
interface
uses Classes, StdCtrls, Graphics, Grids, ClipBrd, Registry;

function Tokenize(S: String; Token: Char): TStringList;
function nextGaussian: Double;
function ReadRegIntAsHexWithDefault(Reg:TRegistry; key:string;default:String): String;
function ReadRegIntWithDefault(Reg:TRegistry; key: string; default: integer): Integer;
function CalculateHistogram(adcCounts:LongWord; Channel: Byte; resultMemo:TMemo=nil):String;
function FirstNonNumber(S: AnsiString): Integer;
function TintColor(MainColor, TintColor: TColor; AmountOfTint: Single): TColor;
function FormattedStrToInt(s:String):Integer;
function FormattedStrToFloat(s:String):Double;
Function FormattedFloatToStr(v:double):string;
Function FormattedIntToStr(v:Integer):string;
function IntToBin(I, MinBits: LongWord): String;
function BinToNybbledBin(b:string):string;
function UnTokenizeCSV(S:TStringList; delim:Char=','):String;
procedure CopySelectionFromStringGrid(SG: TStringGrid);
procedure TokenizeCSV(S : string; Strings : TStringList; delim:Char=',');
function HexToStr(Hex: String): String;
function StrToSpacedHex(Text: String): String;

type
  THistogramData = record
    NumReadings: LongInt;
    Data: array [Word] of LongInt;
    Avg, RMS: Currency;
    MinCounts, MaxCounts: Word;
  end; // ToneChannelHistogram


implementation
uses SysUtils, Math;

function NybbleToHex(N: Byte): Char;
begin
  N := N and $F;
  if N < 10 then
    Result := Char(Byte('0') + N)
  else
    Result := Char(Byte('A') - 10 + N)
  ;
end;

function HexToNybble(Hex: Char): Byte;
begin
  case Hex of
  '0'..'9': Result := Byte(Hex) - Byte('0');
  'A'..'F': Result := Byte(Hex) - Byte('A') + 10;
  'a'..'f': Result := Byte(Hex) - Byte('a') + 10;
  else      raise Exception.CreateFmt('Invalid hex character "%s"', [ Hex ]);
  end;
end;

function StrToHex(Text: String): String;
var
  I: Integer;

begin
  SetString(Result, nil, Length(Text) * 2);
  for I := 1 to Length(Text) do begin
    Result[I * 2 - 1] := NybbleToHex(Byte(Text[I]) shr 4);
    Result[I * 2]     := NybbleToHex(Byte(Text[I])      );
  end;
end;

function HexToStr(Hex: String): String;
var
  I: Integer;

begin
  SetString(Result, nil, (Length(Hex) + 1) div 2);
  for I := 1 to Length(Result) do
    Result[I] := Char(HexToNybble(Hex[I * 2]) + HexToNybble(Hex[I * 2 - 1]) shl 4)
  ;
end;

function StrToSpacedHex(Text: String): String;
var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex(Byte(Text[I]) shr 4);
    Result[I * 3 - 1] := NybbleToHex(Byte(Text[I])      );
  end;
end;

function Tokenize(S: String; Token: Char): TStringList;
var
  I: Integer;

begin
  try
    Result := TStringList.Create;
    repeat
      I := Pos(Token, S);
      if I = 0 then begin
        Result.Add(S);
        Exit;
      end;
      Result.Add(Copy(S, 1, I - 1));
      S := Copy(S, I + 1, MAXINT);
    until False;
  except
    Result := nil;
  end;
end;

function nextGaussian: Double;
{$J+}
const
  nextNextGaussian: Double = 0;
  haveNextNextGaussian: Boolean = False;
{$J-}

var
  v1, v2, s, multiplier: Double;

begin
   if (haveNextNextGaussian = True) then begin
     haveNextNextGaussian := False;
     Result := nextNextGaussian;
   end
   else begin
     repeat
       v1 := 2 * random - 1;   // between -1.0 and 1.0
       v2 := 2 * random - 1;   // between -1.0 and 1.0
       s := v1 * v1 + v2 * v2;
     until not ((s >= 1) or (s = 0));
     multiplier := sqrt(-2 * ln(s)/s);
     nextNextGaussian := v2 * multiplier;
     haveNextNextGaussian := True;
     Result := v1 * multiplier;
   end
end;

function ReadRegIntAsHexWithDefault(Reg:TRegistry; key:string;default:String): String;
begin
  if Reg.ValueExists(key) then
    Result := IntToHex(Reg.ReadInteger(key), length(default))
  else
    Result := default
  ;
end;

function ReadRegIntWithDefault(Reg:TRegistry; key: string; default: integer): Integer;
begin
  if Reg.ValueExists(key) then
    Result := Reg.ReadInteger(key)
  else
    Result := default;
end;

procedure CopySelectionFromStringGrid(SG: TStringGrid);
var
  R: TGridRect;
  Content: AnsiString;
  Contents: TStringList;
  iCol, iRow: Integer;

begin
  Contents := TStringList.Create;
  R := SG.Selection;
  if R.Right >= SG.ColCount then R.Right := SG.ColCount-1;
  if R.Bottom >= SG.RowCount then R.Bottom := SG.RowCount-1;
  for iRow := R.Top to R.Bottom do begin
    Content := '';
    for iCol := R.Left to R.Right do begin
      if iCol <> R.Left then Content := Content + #9;
      Content := Content + SG.Cells[iCol, iRow];
    end;
    Contents.Add(Content);
  end;
  Clipboard.AsText := Contents.Text;
  Contents.Free;
end;


function IntToBin(I, MinBits: LongWord): String;
  var
    DoneBits: LongWord;
begin
  Result := '';
  DoneBits := 0;
  repeat
    Result := Char(Byte('0') + (I and 1)) + Result;
    I := I shr 1;
    Inc(DoneBits);
  until (I = 0) and (DoneBits >= MinBits);
end;

function IntToBinLittleEndian(I, MinBits: LongWord): String;
  var
    DoneBits: LongWord;
begin
  Result := '';
  DoneBits := 0;
  repeat
    Result := Result + Char(Byte('0') + (I and 1));
    I := I shr 1;
    Inc(DoneBits);
  until (I = 0) and (DoneBits >= MinBits);
end;

function BinToNybbledBin(b:string):string;
  var
    nybble: integer;
begin
  for nybble := 0 to (length(b) div 4) do
    Result := Result + copy(b,nybble*4+1, 4) + ' ';
end;

{
function CalculateHistogram
takes 
  an adc count value (0-FFFF)
  a channel number (0-255)
  [optional] output memo
purpose  
  Builds and calculates a histogram. optionally outputs to specified memo
returns 
  a string with a formatted, multi-line histogram result
  
	procedure THistogramForm.Timer1Timer(Sender: TObject);
	var
	  buf: array[0..1] of Word;
	  err: LongWord;
	  bsize: LongWord;

	begin
	  bsize := sizeof(buf);
	  err := GenericVendorRead(diOnly, $BF, 0, 0, bsize, @(buf[0]));
	  memo1.Lines.Clear;
	  memo1.text := CalculateHistogram(buf[0], 0);
	  memo2.Lines.Clear;
	  memo2.text := CalculateHistogram(buf[1], 1);
	  Memo3.lines.clear;
	  memo3.lines.add(inttostr(err));
	end;
}

var
  Histogram: array of THistogramData;

function CalculateHistogram(adcCounts:LongWord; Channel: Byte; resultMemo:TMemo=nil):String;
  var
    rel: Double;
    I, L, NextIndex: Integer;
    StrBuf: array of String;
    OneBuf, HeaderBuf: String;
begin
  Result := '';
  if (channel+1) > Length(Histogram) then // first time this channel has been seen
  begin
    SetLength(Histogram, Channel+1);
    Histogram[Channel].MaxCounts := 0; Histogram[Channel].MinCounts := $FFFF;
  end;

  with Histogram[Channel] do
  begin
    Inc(Data[adcCounts]);
    MaxCounts := Max(MaxCounts, adcCounts);
    MinCounts := Min(MinCounts, adcCounts);

    inc(NumReadings);

    Avg := 0;
    for I := MinCounts to MaxCounts do
      Avg := Avg + Data[I] * I
    ;
    Avg := Avg / NumReadings;

    RMS := 0;
    for I := MinCounts to MaxCounts do
      RMS := RMS + Data[I] * Sqr(I - Avg)
    ;
    RMS := RMS / NumReadings;
    RMS := SqRt(RMS);

    HeaderBuf := format('Max:%.4X Min:%.4X Bins:%Xh RMS:%.3gd Samples:%d'#13#10,[MaxCounts, MinCounts, MaxCounts-MinCounts+1, RMS, NumReadings]);

    //Load data array into display array, and calculate length for display
    //string.
    SetLength(StrBuf, MaxCounts - MinCounts + 1);
    L := Length(HeaderBuf);
    for i := MinCounts to MaxCounts do begin
      if Data[i] <> 0 then begin
        rel := (Data[i] / NumReadings) * 100;
        StrBuf[I - MinCounts] := format('%.4x:%5d      %5.2f%%'#13#10,[ i, Data[i], rel ]);
      end
      else begin
        StrBuf[I - MinCounts] := format('%.4x'#13#10,[ i ]);
      end;
      Inc(L, Length(StrBuf[I - MinCounts]));
    end;//for
    //Load display array into display string.
    SetLength(OneBuf, L);
    NextIndex := 1;
    Move(HeaderBuf[1], OneBuf[NextIndex], Length(HeaderBuf));
    Inc(NextIndex, Length(HeaderBuf));
    for I := 0 to High(StrBuf) do begin
      Move(StrBuf[I][1], OneBuf[NextIndex], Length(StrBuf[I]));
      Inc(NextIndex, Length(StrBuf[I]));
    end;
    SetLength(StrBuf, 0);
    if resultMemo <> nil then
      resultMemo.Lines.Text := OneBuf;
    Result := OneBuf;
    OneBuf := '';

  end;//with histogram[channel]
end;

function FirstNonNumber(S: AnsiString): Integer;
begin
  for Result := 1 to Length(S) do
    if (S[Result] < '0') or (S[Result] > '9') then
      Exit
  ;
  Result := 0;
end;

type
    TRGBFColor = packed record
      case Boolean of
      True: (
        Color: TColor;
      );
      False: (
        Red: Byte;
        Green: Byte;
        Blue: Byte;
        Flags: Byte;
      );
    end;


function TintColor(MainColor, TintColor: TColor; AmountOfTint: Single): TColor;
  var
    FC: TRGBFColor;
    BC: TRGBFColor;
    RC: TRGBFColor absolute Result;
    ANess: Single;
begin
  FC.Color := ColorToRGB(MainColor);
  BC.Color := ColorToRGB(TintColor);
  ANess := 1 - AmountOfTint;
  RC.Red   := Round(ANess * FC.Red   + AmountOfTint * BC.Red  );
  RC.Green := Round(ANess * FC.Green + AmountOfTint * BC.Green);
  RC.Blue  := Round(ANess * FC.Blue  + AmountOfTint * BC.Blue );
  RC.Flags := 0;
end;

function FormattedStrToInt(s:String):Integer;
  var
    iSrc, iDst: integer;
    d: string;
    foo: Integer;
begin
  if length(s) > 0 then
  begin
    SetLength(d, 255);
    iDst := 1;
    for iSrc := 1 to Length(s) do begin
      if s[iSrc] = ',' then continue;
      d[iDst] := s[iSrc];
      inc(iDst);
    end;
    SetLength(d,iDst-1);
    foo := 0;
    try
      foo := StrToInt(d);
    except end;
    Result := foo;
  end else begin
    Result := 0;
  end;
end;

function FormattedStrToFloat(s:String):Double;
  var
    foo: Double;
    iSrc, iDst: integer;
    d: string;
begin
  SetLength(d,255);
  iDst := 1;
  for iSrc := 1 to Length(s) do begin
    if s[iSrc] = ',' then continue;
    d[iDst] := s[iSrc];
    inc(iDst);
  end;
  SetLength(d,iDst-1);
  foo := 0.0;
  try
    foo := StrToFloat(d);
  except end;
  result := foo;
end;

Function FormattedFloatToStr(v:double):string;
begin
  Result := FloatToStrF(v, ffFixed, 7, 2); // change to insert friendly commas
end;

Function FormattedIntToStr(v:Integer):string;
begin
  Result := IntToStr(v); // change to insert friendly commas
end;


procedure TokenizeCSV(S : string; Strings : TStringList; delim:Char);
  var
    C : Integer;
    L : Integer;
    Field : string;
    Quote : boolean;
begin
  L := Length(S);
  Strings.Clear;
  if L > 0 then
  begin
    C := 1;
    repeat
      Field := '';
      if S[C] = '"' then
        begin
                Inc(C);
                repeat
                  while S[C] <> '"' do
                  begin
                    Field := Field + S[C];
                    Inc(C);
                  end;
                  Quote := False;
                  while (C < L) and (S[C] = '"') do
                  begin
                    Inc(C);
                    if Quote then Field := Field + '"';
                    Quote := not Quote;
                  end;
                until (C >= L) or Quote;
        end
        else
        begin
                while (C <= L) do
                begin
                    if (S[C] <> delim) then
                    begin
                      Field := Field + S[C];
                      Inc(C);
                    end else break;  
                end;
        end;
        Strings.Add(Field);
        Inc(C);
    until C >= L;
    Strings.Add('');
  end;
end;

function UnTokenizeCSV(S: TStringList; delim:Char): String;
begin
  Result := StringReplace( Trim(S.Text), #13#10, delim, [rfReplaceAll]);
end;

end.



// #define testbit(b, ibit) ((b & (1<<ibit)) ? 1 : 0)  // bool bit set
