unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    DeviceMemo: TMemo;
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    memoDebug: TMemo;
    Panel2: TPanel;
    butClose: TButton;
    butDump: TButton;
    Bevel1: TBevel;
    Panel3: TPanel;
    PNPCheck: TCheckBox;
    FirmCheck: TCheckBox;
    SNCheck: TCheckBox;
    UserCheck: TCheckBox;
    butErase: TButton;
    CalRadios: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure butCloseClick(Sender: TObject);
    procedure butEraseClick(Sender: TObject);
    procedure butDumpClick(Sender: TObject);
  private
    procedure EraseEEPROM(Start, Len: Word);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB, J2HUtils;

function NybbleToHex(N: Byte): Char;
begin
  N := N and $F;
  if N < 10 then
    Result := Char(Byte('0') + N)
  else
    Result := Char(Byte('A') - 10 + N)
  ;
end;

function MBox(Text, Caption: String; Flags: LongWord): LongWord;
begin
  Result := MessageBox(0, PChar(Text), PChar(Caption), Flags);
  Text := '';
  Caption := '';
end;

type
  TAIOUSB_CloseDevice = function (DeviceIndex: LongWord): LongWord; cdecl;

var
  AIOUSB_CloseDevice: TAIOUSB_CloseDevice;

type
  TDeviceData = record
    DI: LongWord;
    PID: LongWord;
    CalChannels: Integer;
    Nom: String;
    SN: String;
    EMode: String;
  end;

const
  EMode00 = '00 (erased)';
  EModeC0 = 'C0 (plain)';
  EModeC2 = 'C2 (firmware)';

var
  Dev: TDeviceData;


function SNToStr(SN: Int64): String;
var
  I: Integer;
  
begin
  Result := 'XXXX-XXXX-XXXX-XXXX';
  for I := Length(Result) downto 1 do
    if Result[I] = 'X' then begin
      Result[I] := NybbleToHex(SN);
      SN := SN shr 4;
    end
  ;
end;

function EModeToStr(EMode: Byte): String;
begin
  if EMode = $C0 then
    Result := EModeC0
  else if EMode = $C2 then
    Result := EModeC2
  else if EMode = $00 then
    Result := EMode00
  else
    Result := IntToHex(EMode, 2) + ' (???)'
  ;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  Status, DI, BufSize: LongWord;
  Buf: String;
  SN: Int64;
  EMode: Byte;

begin
  PNPCheck.Align := alTop;
  FirmCheck.Align := alTop;
  SNCheck.Align := alTop;
  UserCheck.Align := alTop;
  CalRadios.Align := alTop;
  butErase.Align := alTop;

  butClose.Align := alLeft;
  butDump.Align := alClient;

  repeat
    DI := ResolveDeviceIndex(diOnly);
    if DI <> diNone then Break;

    Status := GetLastError;
    case Status of
      ERROR_FILE_NOT_FOUND: if MBox('No ACCES USB devices found. Plug one in to erase it.', 'EClear - No Device Found', MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
      ERROR_DUP_NAME: if MBox('Too many ACCES USB devices found; this program only erases one at a time.', 'EClear - Too Many Devices', MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
      else if MBox('Error ' + IntToStr(Status) + ' detecting ACCES USB devices.', 'EClear - Error ' + IntToStr(Status), MB_RETRYCANCEL) <> IDRETRY then begin
        Application.ShowMainForm := False;
        Application.Terminate;
        Exit;
      end;
    end;
    if Assigned(AIOUSB_CloseDevice) then
      for I := 0 to 31 do AIOUSB_CloseDevice(I)
    ;
    GetDevices;
  until False;

  Dev.DI := DI;

  BufSize := MAX_PATH;
  SetString(Buf, nil, BufSize);
  QueryDeviceInfo(DI, @Dev.PID, @BufSize, @Buf[1], nil, nil);
  SetLength(Buf, BufSize);
  Dev.Nom := Buf;

  case Dev.PID of
    $8060,
    $8070, $8071, $8078, $8079: Dev.CalChannels := 16;
    $8072, $8073, $807A, $807B: Dev.CalChannels := 12;
    $8074, $8075, $807C, $807D: Dev.CalChannels := 8;
    $8076, $8077, $807E, $807F: Dev.CalChannels := 4;
    else Dev.CalChannels := 0;
  end;
  CalRadios.Visible := Dev.CalChannels <> 0;

  GetDeviceSerialNumber(DI, SN);
  Dev.SN := SNToStr(SN);

  BufSize := 8;
  SetString(Buf, nil, BufSize);
  GenericVendorRead(DI, $A2, $0000, 0, BufSize, @Buf[1]);

  EMode := $FF;
  BufSize := SizeOf(EMode);
  GenericVendorRead(DI, $A2, $0000, 0, BufSize, @EMode);
  Dev.EMode := EModeToStr(EMode);

  Application.ProcessMessages;

  DeviceMemo.Lines.Insert(0, 'Found ' + Dev.Nom);
  DeviceMemo.Lines.Insert(1, 'SN: ' + Dev.SN);
  DeviceMemo.Lines.Insert(2, 'EEPROM Mode: ' + Dev.EMode);
  DeviceMemo.Lines.Insert(3, '');

  if ParamCount > 0 then memoDebug.Visible := True;
end;

procedure TForm1.butCloseClick(Sender: TObject);
begin
  Close;
end;

{ EraseEEPROM()
  Performs a compliant erase of volatile memory by
    1: overwrite with random data
    2: overwrite with all zeros
    3: ovewrite with all ones
  It then overwrites with all zeros, again, to put the device in "factory state" }
procedure TForm1.EraseEEPROM(Start, Len: Word);
var
  Buf: String;

    function randomString( Start, Len: Word): String;
      var
        i: Word;

    begin
      Randomize;
      Result := '';
      for i := 1 to Len do begin
        Result := Result + chr(Random(256));
      end;
      if memoDebug.Visible = TRUE then
        memoDebug.Lines.Add( '@' + IntToHex(Start, 4) + ' := ' + StrToSpacedHex(Result) );
    end;

begin
  // overwrite with random data
  Buf := randomString( Start, Len );
  DeviceMemo.Lines.Add(' ... pass 1 of 4 (random)');
  GenericVendorWrite(Dev.DI, $A2, Start, 0, Length(Buf), @Buf[1]);
  Application.ProcessMessages;

  // overwrite with all zeros
  Buf := StringOfChar(#$00, Len);
  DeviceMemo.Lines.Add(' ... pass 2 of 4 (zeros)');
  GenericVendorWrite(Dev.DI, $A2, Start, 0, Length(Buf), @Buf[1]);
  Application.ProcessMessages;

  // overwite with all ones
  Buf := StringOfChar(#$FF, Len);
  DeviceMemo.Lines.Add(' ... pass 3 of 4 (ones)');
  GenericVendorWrite(Dev.DI, $A2, Start, 0, Length(Buf), @Buf[1]);
  Application.ProcessMessages;

  // overwite with all zero
  Buf := StringOfChar(#$00, Len);
  DeviceMemo.Lines.Add(' ... pass 4 of 4 (factory state)');
  GenericVendorWrite(Dev.DI, $A2, Start, 0, Length(Buf), @Buf[1]);
end;

procedure TForm1.butEraseClick(Sender: TObject);
var
  Buf: String;
  I: Integer;
  EndTime: LongWord;
  dT: LongInt;

begin
  if memoDebug.Visible then memoDebug.Lines.Clear;

  if PNPCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing PnP...');
    EraseEEPROM(0, $8);
    Dev.EMode := EModeToStr($00);
    DeviceMemo.Lines[2] := 'EEPROM Mode: ' + Dev.EMode;
  end;

  if FirmCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing Firmware...');
    EraseEEPROM($8, $1DE8);

    // if the firmware is erased the EMode must change to "soft" or you brick the unit
    if (Dev.EMode = EModeC2) and not PNPCheck.Checked then begin
      Buf := #$C0;
      GenericVendorWrite(Dev.DI, $A2, $0000, 0, Length(Buf), @Buf[1]);
      Dev.EMode := EModeToStr($C0);
      DeviceMemo.Lines[2] := 'EEPROM Mode: ' + Dev.EMode;
    end;
  end;

  if SNCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing SN & Hashes...');
    EraseEEPROM($1DF0, $10);
    Dev.SN := SNToStr(0);
    DeviceMemo.Lines[1] := 'SN: ' + Dev.SN;
  end;

  if UserCheck.Checked then begin
    DeviceMemo.Lines.Add('Erasing User Area...');
    EraseEEPROM($1E00, $200);
  end;

  if CalRadios.Visible then
    //The flash is structured differently from the EEPROM.
    case CalRadios.ItemIndex of
      0: ;
      1: begin
        DeviceMemo.Lines.Add('Erasing DAC Calibration Area...');
        //Quick erase all calibration flash.
        for I := 0 to 31 do
          AIOUSB_FlashEraseSector(Dev.DI, I)
        ;
      end;
      2: for I := 0 to 15 do
        if I < Dev.CalChannels then begin
          //The DAc exists, so uncalibrate it.
          DeviceMemo.Lines.Add('Uncalibrating DAC ' + IntToStr(I) + '...');
          DACSetChannelCal(Dev.DI, I, '1TO1');
          EndTime := GetTickCount + 4000;
          repeat
            Application.ProcessMessages;
            dT := GetTickCount - EndTime; //Using a signed delta to handle wrap.
          until dT > 0;
        end
        else begin
          if I = Dev.CalChannels then DeviceMemo.Lines.Add('Quick Erasing Unused DAC Calibration...');
          //There's no DAC on that channel, so just quick erase its flash.
          AIOUSB_FlashEraseSector(Dev.DI, I * 2);
          AIOUSB_FlashEraseSector(Dev.DI, I * 2 + 1);
        end
      ;
    end
  ;

  DeviceMemo.Lines.Add('Done.');
end;


var
  hAIOUSB: THandle;

procedure TForm1.butDumpClick(Sender: TObject);
var
  Buf: String;
  Status, L: Longword;

begin
  memoDebug.Visible := True;
  L := $2000; // note: someday we hope this will be larger! --J2H "to maintain"
  SetLength(Buf, L);
  Status := GenericVendorRead(Dev.DI, $A2, $0000, 0, L, @buf[1]);
  if Status = ERROR_SUCCESS then
    memoDebug.Lines.Add( StrToSpacedHex(buf))
  else
    memoDebug.Lines.Add( 'error ' + IntToHex(Status, 2) + ' during dump');

end;

initialization
  hAIOUSB := LoadLibrary('AIOUSB.dll');
  AIOUSB_CloseDevice := GetProcAddress(hAIOUSB, 'AIOUSB_CloseDevice');
finalization
  FreeLibrary(hAIOUSB);
  hAIOUSB := 0;
end.
